//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "citiesreport.h"
#include "armysetlist.h"
#include "playerlist.h"
#include <stdio.h>

//first comes the implementation of CityItem. A CityItem is a bar of ~420x80
//pixels which displays infos about a single city.
class CityItem : public SigC::Object, public PG_EventObject, public PG_Widget
{
    public:
        CityItem(PG_Widget* parent, PG_Rect rect);
        ~CityItem();

        void readData(City* city);

        SigC::Signal1<void, City*> sclicked;

    private:
        bool eventMouseButtonDown(const SDL_MouseButtonEvent * event);

        PG_Label* d_l_name;
        PG_Label* d_l_owner;
        PG_Label* d_l_gold;
        PG_Label* d_l_production;
        PG_Label* d_l_finished;
        PG_Label* d_l_noinfo;
        PG_Button* d_b_production;
        City* d_city;
};

CityItem::CityItem(PG_Widget* parent, PG_Rect rect)
    :PG_Widget(parent, rect)
{
    d_l_name = new PG_Label(this, PG_Rect(10, 10, 150, 20), "");
    d_l_owner = new PG_Label(this, PG_Rect(10, 40, 150, 20), "");
    d_l_gold =new PG_Label(this, PG_Rect(10, 60, 100, 20), "");
    d_l_production = new PG_Label(this, PG_Rect(300, 20, 100, 20), "");
    d_l_finished = new PG_Label(this, PG_Rect(300, 50, 100, 20), "");
    d_b_production = new PG_Button(this, 0, PG_Rect(220, 10, 55, 55), "");
    d_b_production->EnableReceiver(false);
	d_b_production->Hide();
    d_l_noinfo = new PG_Label(this,PG_Rect(300, 20, 150, 50), "no information");
	d_l_noinfo->Hide();
    d_l_noinfo->SetFontColor(200, 100, 100);
    d_l_noinfo->SetFontSize(16);
}

CityItem::~CityItem()
{
}

void CityItem::readData(City* city)
{
    d_city = city;
	
    d_l_name->SetText(city->getName().c_str());
	SDL_Color color = city->getPlayer()->getColor();
    d_l_name->SetFontColor(color.r, color.g, color.b);
	
    d_l_owner->SetTextFormat("owned by %s", city->getPlayer()->getName().c_str());
    d_l_gold->SetTextFormat("gold: %i", city->getGold());

    if (city->getPlayer() == Playerlist::getInstance()->getActiveplayer())
    {
        int armytype = city->getArmytype(city->getProduction());
        if (armytype == -1)
        //i.e. no production selected
        {
            d_b_production->SetIcon((SDL_Surface*)0, 0, 0);
            d_b_production->SetText("N");
            d_b_production->SetFontSize(20);
            d_l_production->SetText("None");
            d_l_finished->SetText("n.a.");
        }
        else
        {
            Armyset* as = Armysetlist::getInstance()->getArmyset(city->getPlayer()->getArmysetName());
            Armyset_army* a = &(as->army[armytype]);

            d_b_production->SetText("");
            d_b_production->SetIcon(a->getPixmap(), 0, 0);
            d_l_production->SetText(a->getName().c_str());
            d_l_finished->SetTextFormat("%i", city->getDuration());
        }

        d_b_production->Show();
        d_l_production->Show();
        d_l_finished->Show();
        d_l_noinfo->Hide();
    }
    else    //city belongs to another player, so don't show info about it
    {
        d_b_production->Hide();
        d_l_production->Hide();
        d_l_finished->Hide();
        d_l_noinfo->Show();
    }
}

bool CityItem::eventMouseButtonDown(const SDL_MouseButtonEvent* event)
{
    if (!IsVisible())
        return true;

    sclicked.emit(d_city);
    return true;
}


//now comes the CitiesReport class
CitiesReport::CitiesReport(PG_Widget* parent, PG_Rect rect)
    :PG_Window(parent, rect, "city report", WF_MODAL), d_index(0)
{
    //I assume 500x400 pixels here as well
    d_b_ok = new PG_Button(this, 2, PG_Rect(450, 370, 45, 20), "OK");
    d_b_ok->SetEventObject(MSG_BUTTONCLICK, this,
                            (MSG_CALLBACK_OBJ)&CitiesReport::b_okClicked);

    d_b_up = new PG_Button(this, 0, PG_Rect(450, 150, 45, 20), "Prev.");
    d_b_up->SetEventObject(MSG_BUTTONCLICK, this,
                            (MSG_CALLBACK_OBJ)&CitiesReport::b_upClicked);

    d_b_down = new PG_Button(this, 1, PG_Rect(450, 250, 45, 20), "Next");
    d_b_down->SetEventObject(MSG_BUTTONCLICK, this,
                            (MSG_CALLBACK_OBJ)&CitiesReport::b_downClicked);

    d_l_number = new PG_Label(this, PG_Rect(460, 35, 45, 20), "");

    d_items[0] = new CityItem(this, PG_Rect(10, 30, 420, 80));
    d_items[1] = new CityItem(this, PG_Rect(10, 120, 420, 80));
    d_items[2] = new CityItem(this, PG_Rect(10, 210, 420, 80));
    d_items[3] = new CityItem(this, PG_Rect(10, 300, 420, 80));

    //now get all cities sorted by player with the activeplayer first and
    //the cities of the neutral player last
    Playerlist::iterator pit;
    Citylist::iterator cit;

    d_citylist.resize(Citylist::getInstance()->size());
    //first, cities of the active player
    for (cit = Citylist::getInstance()->begin();
        cit != Citylist::getInstance()->end(); cit++)
    {
        if ((*cit).getPlayer() == Playerlist::getInstance()->getActiveplayer())
        {
            d_citylist[d_index] = &(*cit);
            d_index++;
        }
    }

    //now the other non-neutral players
    for (pit = Playerlist::getInstance()->begin();
        pit != Playerlist::getInstance()->end(); pit++)
    {
        if (((*pit) == Playerlist::getInstance()->getActiveplayer())
            || ((*pit) == Playerlist::getInstance()->getNeutral()))
        {
            continue;
        }

        for (cit = Citylist::getInstance()->begin();
            cit != Citylist::getInstance()->end(); cit++)
        {
            if ((*cit).getPlayer() == (*pit))
            {
                d_citylist[d_index] = &(*cit);
                d_index++;
            }
        }
    }

    //now the neutral player
    for (cit = Citylist::getInstance()->begin();
        cit != Citylist::getInstance()->end(); cit++)
    {
        if ((*cit).getPlayer() == Playerlist::getInstance()->getNeutral())
        {
            d_citylist[d_index] = &(*cit);
            d_index++;
        }
    }

    d_index = 0;

    fillCityItems();

}

CitiesReport::~CitiesReport()
{
    d_citylist.clear();
}

PARAGUI_CALLBACK(CitiesReport::b_okClicked)
{
    SendMessage(this, MSG_MODALQUIT, 0, 0);
    return true;
}

PARAGUI_CALLBACK(CitiesReport::b_upClicked)
{
    d_index--;
    if(d_index < 0)
    d_index = 0;

    fillCityItems();

    return true;
}

PARAGUI_CALLBACK(CitiesReport::b_downClicked)
{
    d_index++;
    if ((d_index+4) > d_citylist.size())
        d_index = d_citylist.size() - 4;

    fillCityItems();

    return true;
}

void CitiesReport::fillCityItems()
{
    d_l_number->SetTextFormat("%i - %i", d_index+1, d_index + 4);

    for (int i = 0; i < 4; i++)
    {
        d_items[i]->readData(d_citylist[d_index+i]);
        d_items[i]->Redraw();
    }
}
